from django.db.models import Count
from netbox.views import generic
from . import models, tables, forms
from ipam.models import VLAN
from ipam.utils import add_available_vlans
from ipam.tables import VLANTable


class VLANGroupSetView(generic.ObjectView):
    queryset = models.VLANGroupSet.objects.all()

    def get_extra_context(self, request, instance):
        vlan_groups = instance.vlan_groups.all()
        max_vid = max(vlan_groups, key=(lambda x: x.max_vid)).max_vid
        min_vid = min(vlan_groups, key=(lambda x: x.min_vid)).min_vid

        vlan_group_vlans = {}
        for vid in range(min_vid, max_vid + 1):
            vlan_group_vlans[vid] = {}
            for vlan_group in vlan_groups:
                vlans = VLAN.objects.filter(group=vlan_group, vid=vid)
                vlan_group_vlans[vid][vlan_group.id] = vlans[0] if vlans else None

        return {
            'vlan_group_vlans': vlan_group_vlans
        }


class VLANGroupSetListView(generic.ObjectListView):
    queryset = models.VLANGroupSet.objects.annotate(
        vlan_group_count=Count('vlan_groups')
    )
    table = tables.VLANGroupSetTable


class VLANGroupSetEditView(generic.ObjectEditView):
    queryset = models.VLANGroupSet.objects.all()
    form = forms.VLANGroupSetForm


class VLANGroupSetDeleteView(generic.ObjectDeleteView):
    queryset = models.VLANGroupSet.objects.all()



from rest_framework import serializers
from netbox.api.serializers import NetBoxModelSerializer
from ..models import VLANGroupSet


class VLANGroupSetSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name=f'plugins-api:netbox_vlan_manager-api:vlangroupset-detail'
    )
    vlan_group_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = VLANGroupSet
        fields = (
            'id',
            'url',
            'display',
            'name',
            'description',
            'comments',
            'tags',
            'custom_fields',
            'created',
            'last_updated',
            'vlan_group_count',
        )



import django_tables2 as tables
from netbox.tables import NetBoxTable
from .models import VLANGroupSet


class VLANGroupSetTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    vlan_group_count = tables.Column()

    class Meta(NetBoxTable.Meta):
        model = VLANGroupSet
        fields = (
            'pk',
            'id',
            'name',
            'vlan_group_count',
            'description',
            'comments'
        )
        default_columns = (
            'name',
            'vlan_group_count',
            'description'
        )



from django.db.models import Count
from netbox.api.viewsets import NetBoxModelViewSet
from .serializers import VLANGroupSetSerializer
from .. import models


class VLANGroupSetListViewSet(NetBoxModelViewSet):
    queryset = models.VLANGroupSet.objects.prefetch_related('tags').annotate(
        vlan_group_count=Count('vlan_groups')
    )
    serializer_class = VLANGroupSetSerializer



from extras.plugins import PluginConfig
from .version import __version__


class VLANManager(PluginConfig):
    name = 'netbox_vlan_manager'
    verbose_name = 'NetBox VLAN Manager'
    description = 'NetBox VLAN Manager for multitple VLAN Groups'
    author = 'miyuk'
    author_email = 'miyuk@miyuk.net'
    version = __version__
    base_url = 'vlan-manager'


config = VLANManager



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel


class VLANGroupSet(NetBoxModel):
    name = models.CharField(
        max_length=100,
        unique=True
    )
    vlan_groups = models.ManyToManyField(
        to='ipam.VLANGroup',
        related_name='vlan_group_sets',
        blank=True
    )
    description = models.CharField(
        max_length=200,
        blank=True
    )
    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse(f'plugins:netbox_vlan_manager:vlangroupset', kwargs={'pk': self.pk})



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'netbox_vlan_manager'

router = NetBoxRouter()
router.register('vlan-group-sets', views.VLANGroupSetListViewSet)

urlpatterns = router.urls
